﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Configuration;
using System.Collections;
using System.Xml.Linq;
using System.Text.RegularExpressions;
using System.ServiceModel.Syndication;
using System.Xml;
using MonoSoftware.MonoX;
using System.Web.UI.WebControls.WebParts;
using MonoSoftware.Web;
using MonoSoftware.MonoX.Caching;
using MonoSoftware.MonoX.Utilities;
using MonoSoftware.MonoX.Repositories;
using MonoSoftware.MonoX.DAL.EntityClasses;
using MonoSoftware.MonoX.DAL.HelperClasses;
using NVelocity;
using System.IO;


namespace DocumentDownload
{
    /// <summary>
    /// Lists all the files for download with folder path and file extension specified. NOTE that this module is not taking care of the folder and file permissions.
    /// </summary>
    [WebPartCatalogCategory("Samples")]
    public partial class DocumentDownloadModule : BaseAutoRegisterPart
    {
        #region Fields
        private const string AllFilesExtension = "*.*";
        private const string Root = "~/";
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets files count.
        /// </summary>
        [WebBrowsable(true), Personalizable(true)]
        [WebDescription("File display count")]
        [WebDisplayName("File display count")]
        public Int32? FileCount { get; set; }

        private string _allowedExtension = AllFilesExtension;
        /// <summary>
        /// If set, files with this extension will be displayed.
        /// </summary>
        [WebBrowsable(true), Personalizable(true)]
        [WebDescription("Allowed extension")]
        [WebDisplayName("Allowed extension")]
        public string AllowedExtension
        {
            get { return _allowedExtension; }
            set { _allowedExtension = value; }
        }

        private string _folderPath = Root;
        /// <summary>
        /// Folder from which the files will be displayed.
        /// </summary>
        [WebBrowsable(true), Personalizable(true)]
        [WebDescription("Folder path")]
        [WebDisplayName("Folder path")]
        public string FolderPath
        {
            get { return _folderPath; }
            set { _folderPath = value; }
        }

        #endregion

        #region Constructor
        /// <summary>
        /// Constructor.
        /// </summary>
        public DocumentDownloadModule()
        {
            Title = "Document download module";
            IsTemplated = true;
            ControlSpecificTemplateSubPath = "DocumentDownloadTemplates";
        }
        #endregion

        #region Page Events
        protected override void OnInit(EventArgs e)
        {            
            this.lvItems.ItemDataBound += new EventHandler<ListViewItemEventArgs>(lvItems_OnItemDataBound);
            
            base.OnInit(e);           
        }

        protected override void OnLoad(EventArgs e)
        {
            DataBind();
            base.OnLoad(e);            
        }
        #endregion

        #region Methods
        /// <summary>
        /// Apply web part property changes (Note: Overridden property still needs to be marked as <see cref="MonoSoftware.MonoX.WebPartApplyChangesAttribute"/>).
        /// <para>
        /// Note: Marked with <see cref="MonoSoftware.MonoX.WebPartApplyChangesAttribute"/> attribute so it is called from ApplyChanges event in the editor part to refresh the module appearance.
        /// </para>
        /// </summary>
        [WebPartApplyChanges]
        public override void ApplyChanges()
        {
            base.ApplyChanges();
            DataBind();
        }
        /// <summary>
        /// Data binding.
        /// </summary>
        public override void DataBind()
        {
            if (!String.IsNullOrEmpty(FolderPath))
            {
                DirectoryInfo dirInfo = new DirectoryInfo(Server.MapPath(FolderPath));
                
                FileInfo[] fileInfos = null;
                try
                {
                    if (AllowedExtension.Equals(AllFilesExtension) || AllowedExtension.Equals(String.Empty))
                        fileInfos = dirInfo.GetFiles();
                    else
                        fileInfos = dirInfo.GetFiles(AllowedExtension);
                }
                catch
                {
                    lblMessage.Visible = true;
                    lblMessage.Text = "An error has occured. Please check the folder path.";
                    return;
                }

                if (fileInfos.Count() > 0)
                {
                    if (FileCount.HasValue)
                        lvItems.DataSource = fileInfos.Take(FileCount.Value);
                    else
                        lvItems.DataSource = fileInfos;
                    lvItems.DataBind();
                }
            }

        }

        protected void lvItems_OnItemDataBound(object sender, ListViewItemEventArgs e)
        {
            if (e.Item.ItemType == ListViewItemType.DataItem)
            {
                FileInfo listItem = ((ListViewDataItem)e.Item).DataItem as FileInfo;
                Hashtable tags = ParseTemplateTags(listItem);
                VelocityContext velocityContext = new VelocityContext();
                velocityContext.Put(ItemDataBoundEventArgs.ContainerParamName, new ItemDataBoundEventArgs(((ListViewDataItem)e.Item).DataItemIndex, listItem));

                //Note: Below code is for MonoX demo compatibility, it is not needed in case you are using the Twitter module separately from the MonoX demo.
                string template = CurrentTemplateHtml;
               /* if (String.IsNullOrEmpty(template))
                    try
                    {
                        template = System.IO.File.ReadAllText(HttpContext.Current.Server.MapPath(Paths.MonoX.Samples.TwitterFeedModule.App_Templates.Default.TwitterFeedTemplates.Default_htm));
                    }
                    catch (Exception ex)
                    {
                        LogUtility log = new LogUtility(Guid.NewGuid().ToString());
                        log.Error(Paths.MonoX.Samples.TwitterFeedModule.App_Templates.Default.TwitterFeedTemplates.Default_htm, ex);
                    }*/
                RenderTemplatedPart(e.Item, String.Empty, template, tags, velocityContext);

                LinkButton lnkDownload = e.Item.FindControl("lnkDownload") as LinkButton;
                if (lnkDownload != null)
                {
                    if (ScriptManager.GetCurrent(this.Page) != null)
                        ScriptManager.GetCurrent(this.Page).RegisterPostBackControl(lnkDownload);
                    lnkDownload.Click += new EventHandler(lnkDownload_Click);
                }
            }
        }

        protected virtual Hashtable ParseTemplateTags(FileInfo listItem)
        {
            Hashtable tags = new Hashtable(StringComparer.InvariantCultureIgnoreCase);
            
            LinkButton item = new LinkButton();
            item.ID = "lnkDownload";
            item.CommandArgument = FolderPath + listItem.Name;
            item.Text = listItem.Name;
            tags["<# Item #>"] = item;

            tags["<# DateCreated #>"] = "Date created: " + listItem.CreationTimeUtc.ToShortDateString();

            return tags;
        }

        private void HttpResponseFile(HttpResponse response, string fileName, string filePath)
        {
            response.Clear();
            response.AppendHeader("Content-Disposition", String.Format("attachment; filename=\"{0}\"", GetValidFileName(Path.GetFileName(fileName))));
            response.ContentType = String.Format("application/{0}", GetValidFileName(Path.GetExtension(fileName)).Trim('.'));

            using (FileStream fStream = new FileStream(filePath, FileMode.Open, FileAccess.Read))
            {
                byte[] bytes = new byte[4096];
                int n;
                while ((n = fStream.Read(bytes, 0, bytes.Length)) != 0)
                    response.OutputStream.Write(bytes, 0, n);
            }
            response.End();
        }

        private string GetValidFileName(string str)
        {
            List<char> invalidChars = new List<char>(Path.GetInvalidFileNameChars());

            if (!String.IsNullOrEmpty(str))
            {
                foreach (char item in str)
                {
                    if (invalidChars.Contains(item))
                        str = str.Replace(item, char.MinValue);
                }
            }
            return str;
        }

        #endregion

        #region UI Events
        void lnkDownload_Click(object sender, EventArgs e)
        {
            string filePath = Server.MapPath(((LinkButton)sender).CommandArgument);
            HttpResponseFile(Response, Path.GetFileName(filePath), filePath);
        }
        #endregion
    }
}